@@ -549,12 +549,19 @@ void video_output_render(VideoOutput* self) {
     return;
   }
   
-  // H/W rendering
+  // H/W rendering with triple buffering
   if (self->texture_gl && self->render_context) {
-    texture_gl_render(self->texture_gl);
+    // Render to write buffer
+    gboolean rendered = texture_gl_render(self->texture_gl);
     
-    // Notify Flutter that a new frame is available
-    fl_texture_registrar_mark_texture_frame_available(
-        self->texture_registrar, FL_TEXTURE(self->texture_gl));
+    // Only swap and notify if rendering was actually performed
+    if (rendered) {
+      // Publish the rendered frame (update buffer indices)
+      texture_gl_swap_buffers(self->texture_gl);
+      
+      // Notify Flutter that a new frame is available
+      fl_texture_registrar_mark_texture_frame_available(
+          self->texture_registrar, FL_TEXTURE(self->texture_gl));
+    }
   }
 }
